/*******************************************************
 **                                                   **
 **  King's Crown Addon ADDON SOURCE (version 1.44)   **
 **                                                   **
 **  written by:  Jay L. Faries                       **
 **                                                   **
 **  for use with King's Crown version 1.44 only.     **
 **                                                   **
 *******************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <fcntl.h>
#include "opendoor.h"

/************************
 * constant definitions *
 ************************/

#define MAXPL       127           //maximum number of players
#define MAXNODES    8             //maximum number of nodes

typedef unsigned int Uint;
typedef unsigned long Ulong;

/******************************
 * Structures and global data *
 ******************************/

typedef struct
{
	char    name[36];           //characters name  (name[35] is Node #)
								//(name[34] is index of last player attacked)
	Uint    attack_str;         //attack strength (base)
	Uint    defense_str;        //defence strength (base)
	int     hp_max;             //maximum Health Points
	int     hp_cur;             //current Health Points
	int     magic_max;          //maximum magic points
	int     magic_cur;          //current magic points
	Ulong   gold_hand;          //gold in hand
	Ulong   gold_bank;          //gold in bank
	Ulong   experience;          //total experience
	Uint    level;              //experience level
	Uint    lays;               //number of lays
	Uint    weapon;             //weapon Index (uses weapon_name array)
	Uint    armor;              //armor index  (uses armor_name array)
	char    user_name[36];      //user name (read from drop file)
	Uint    sc_light;           //number of light scrolls
	Uint    sc_healing;         //number of healing scrolls
	Uint    sc_map;             //number of map scrolls
	Uint    sc_teleport;        //number of teleport scrolls
	Uint    sc_disarm;          //number of disarm scrolls
	Uint    sc_speed;           //number of speed scrolls
	Uint    sc_invisibility;    //number of invisibility scrolls
	Uint    sc_enchant;         //number of enchant scrolls
	Uint    sc_levitate;        //number of Levetate Scrolls
	Uint    sc_return;          //number of Rune of Return scrolls
	int     dungeon_fights;     //number of dungeon fights left
	int     player_fights;      //number of player fights left
	Uint    sex;                //characters Sex:  0=male  1=female
	Uint    knighted;           //number of times knighted
	char    alive;              //character alive?
	Uint    charm;              //characters charm
	char    dungeon_level;      //current dungeon level (0=none)
	char    dungeon_seed;       //seed of current dungeon
	char    dungeon_xpos;       //horizontal position in dungeon
	char    dungeon_ypos;       //vertical position in dungeon
	char    dungeon_max;        //maximum dungeon level attained so far
	long    last_on;            //days since player played last
	char    condition;          //drunkeness level (uses condition_string)
	char    location;           //0=camped, 1=inn, 2=brothel, >2=online somewhere
	char    flirt_inn;          //0=has not flirted at inn yet today
	char    flirt_player;       //0=has not flirted with other player yet today
	char    heard_bards;        //0=has not heard the bards jam yet today
	int     married_to;         //-1= not married, -2 = married to Billy/Betty
	int     sons;               //number of sons
	int     daughters;          //number of daughters
	char    kings_crown;        //player has the crown?

} USER_CHARACTER;

typedef struct
{
	char    name[36];           //characters name
	Ulong   experience;         //total experience
	Uint    level;              //experience level
	Uint    sex;                //characters Sex:  0=male  1=female
	Uint    knighted;           //number of times knighted
	char    alive;              //character alive?
	char    location;           //0=camped, 1=inn, 2=brothel, 3=online
} WLIST;

char weapon_name[41][21] =
{
	"Bare Hands",           "Whip",                 "Dagger",
	"Quarterstaff",         "War Hammer",           "Morning Star",
	"Short Sword",          "Bastard Sword",        "Long Sword",
	"Two-Handed Sword",     "Whip of Soul-Snaring", "Dagger+1",
	"Staff of Striking",    "Spiritual Hammer",     "Morning Star+1",
	"Short Sword+1",        "Bastard Sword+1",      "Long Sword+1",
	"Two-Handed Sword+1",   "Lightning Whip",       "Dagger+2",
	"Staff of Power",       "Hammer of Giants",     "Morning Star+2",
	"Short Sword+2",        "Bastard Sword+2",      "Long Sword+2",
	"Two-Handed Sword+2",   "Whip of Death",        "Dagger+3",
	"Staff of Kurr",        "Hammer of the Gods",   "Morning Star+3",
	"Short Sword+3",        "Bastard Sword+3",      "Long Sword+3",
	"Two-Handed Sword+3",   "Vorpal Sword",         "Staff of Godly Might",
	"Orb of Kurr",          "Skull of Kurr"
};
Uint weapon_strength[41] =
{
	1,3,5,7,9,11,14,17,20,25,30,35,40,50,60,70,85,100,125,150,
	175,200,250,300,350,400,475,600,700,825,
	975,1125,1300,1500,1750,2000,2250,2500,3000,4000,5000
};
char armor_name[41][21] =
{
	"Bare Nakedness",       "Clothes",              "Shield",
	"Leather",              "Padded Leather",       "Ring Mail",
	"Brigadine",            "Scale Mail",           "Hide Armor",
	"Chain Mail",           "Splint Mail",          "Banded Mail",
	"Bronze Plate Mail",    "Field Plate Mail",     "Full Plate Armor",
	"Leather+1",            "Padded Leather+1",     "Ring Mail+1",
	"Brigadine+1",          "Scale Mail+1",         "Hide Armor+1",
	"Chain Mail+1",         "Splint Mail+1",        "Banded Mail+1",
	"Bronze Plate Mail+1",  "Field Plate Mail+1",   "Full Plate Armor+1",
	"Leather+2",            "Padded Leather+2",     "Ring Mail+2",
	"Brigadine+2",          "Scale Mail+2",         "Hide Armor+2",
	"Chain Mail+2",         "Splint Mail+2",        "Banded Mail+2",
	"Bronze Plate Mail+2",  "Field Plate Mail+2",   "Full Plate Armor+2",
	"Armor of Kurr",        "Gawd Armor"
};
Uint armor_strength[41] =
{
	0,2,3,5,7,9,12,15,18,22,25,30,35,40,50,60,75,80,100,125,
	150,175,225,275,325,350,450,500,600,725,
	875,1025,1200,1300,1550,1750,2000,2250,2750,3000,3500
};
Ulong equipment_cost[41] =
{
	0,10,25,40,65,80,100,250,400,650,800,1000,2500,4000,6500,8000,
	10000,25000,40000,65000,80000,100000,250000,400000,650000,800000,
	1000000,2500000,4000000,6500000,8000000,10000000,25000000,40000000,
	65000000,80000000, 100000000,250000000,400000000,650000000,800000000
};
char condition_string[8][21] =
{
	"Stone Cold Sober",     "Slightly Buzzed",
	"Overly Friendly",      "Quite Rowdy",
	"Invincible",           "Sloppy Drunk",
	"Overcome With Nausea", "Passed Out!"
};
char condition_adjust_strength[8] =
{
	0, 1, 2, 3, 0, -1, -2, -3
};
char condition_adjust_defense[8] =
{
	0, 0, 0, 1, 2, 3, -1, -2
};
char condition_adjust_charm[8] =
{
	0, 2, 3, 1, 0, -1, -2, -3
};
Ulong level_exp[25] =
{
	0,100,200,400,800,1600,3200,6400,12800,25600,51200,102400,204800,409600,
	819200,1638400,3276800,6553600,13107200,26214400,52428800,104857600,
	209715200,419430400,838860800
};

char scroll_string[10][21] =
{
	"Rune of Return","Light","Map","Detect Monster","Healing",
	"Teleport","Disarm Trap","Levitation","Invisibility","Enchanted Weapon"
};
Ulong scroll_price[10] =
{
	100,250,5000,750,1000,3250,1500,750,3500,4000
};

char LocationDescription[17][21]=
{
	"Camp Outside",    "Room at the INN","Room at the Brothel",
	"Log on",          "Main Street",    "Saddoc's Armor",
	"Will's Weapons",  "Inn of Drinking","Mordrin's Magic",
	"Arena of Death",  "Post Office",    "Edna's Brothel",
	"Bank of Interest","Helga's Healing","Who knows where?",
	"Dungeons of Kurr","Outside the gates"
};
char Hex[16]= "0123456789abcdef";
Ulong NumBase[10] =
{
	1000000000,100000000,10000000,1000000,100000,10000,1000,100,10,1
};

USER_CHARACTER Player[MAXPL];   //define array of player records (127 entries)
char GameDir[81];               //string containing the full game directory
char DropPath[MAXNODES][81];    //contains 8 drop file paths
char SYSTYPE[MAXNODES];         //contains system type for 8 nodes
Uint ActivePlayer,PassedNode,MAINT,RIP,PG;


/***************************
 *   FUNCTION PROTOTYPES   *
 ***************************/



void j_printf(char *fmt, ...);
void j_putUlong(Ulong Number);
void a_putUlong(Ulong Number);
Uint GetUint(char Attrib, int length);
Ulong GetUlong(char Attrib, int length);
Ulong FInputString(char *string,int maxlen);
Uint ReadKCtime(void);
Uint ReadKCplayer(void);
int LoadCurlyCfg(void);
void LoadPlayer(int Index);
void SavePlayer(int Index);
void LoadPlayers(void);
void SavePlayers(void);
void LoadGameDir(int Node);
void AddNews(char *string);
void SendMail(int FIndex, int TIndex, char *Mail, int flirt);
void LoadDropPath(void);
Uint FindPlayer(char *UserName);


void DoMaint(void);
void ShowStats(void);


/******************************************
 *    Example King's Crown Addon Source   *
 ******************************************/

void main(int argc, char *argv[])
{
	int t,U;
	FILE *fp;


	strcpy(od_registered_to,"UNREGISTERED");
	od_registration_key=0L;

	srand(*(int _far *)(0x0040006c));       //randomize via timer

	sscanf(argv[1],"%d",&PassedNode);

	LoadDropPath();

	strcpy(od_control.info_path, DropPath[PassedNode-1]);

	MAINT=FALSE;
	RIP=FALSE;
	PG=FALSE;
	for (t=1;t<=argc;t++)
	{
		if (strcmp(*argv,"/MAINT")==0 || strcmp(*argv,"/maint")==0)
			MAINT=TRUE;
		if (strcmp(*argv,"/RIP")==0 || strcmp(*argv,"/rip")==0)
			RIP=TRUE;
		if (strcmp(*argv,"/PG")==0 || strcmp(*argv,"/pg")==0)
			PG=TRUE;
		argv++;
	}
	if (MAINT)
	{
		DoMaint();
		exit(0);
	}

	LoadPlayers();
	od_init();
	od_clr_scr();
	ReadKCtime();

	od_autodetect(DETECT_NORMAL);

	ReadKCplayer();

	LoadPlayer(ActivePlayer);
	od_control.user_screen_length=25;

	ShowStats();

	j_printf("`n`3-`9=`1`bMORE`1`9=`3-");
	od_get_key(TRUE);
	j_printf("\r          \r");
	j_printf("`eNow returning to King's Crown...`n");
	od_exit(0,0);
}


//    FUNCTION:
//        void DoMaint(void);
//    DESCRIPTION:
//        this function should be called for any daily maintenence your
//        addon requires.  it should be called when there is a /MAINT option
//        present in the command line.
//
//        /MAINT is required and should be accounted for even if your IGM
//        doesn't need to run maintenence.  In this case it should do nothing
//        and just exit (do nothing) if the /MAINT parameter is detected.
//        If your IGM needs maintenence the it should only run maintenence when
//        /MAINT is detected.
//    RETURNS:
//        nothing
void DoMaint(void)
{
	od_exit(0,0);
}


//    FUNCTION:
//        void ShowStats(void);
//    DESCRIPTION:
//        this function demonstrates some of the uses of the the functions
//        contained in this library.  It displays the user record indexed by
//        ActivePlayer.  Note the formulas for Attack and Defense Strength,
//        they are modified by the condition_adjust... values.
//    RETURNS:
//        nothing
void ShowStats(void)
{
	Uint temp;

	od_clr_scr();

	LoadPlayer(ActivePlayer);
	j_printf("`t`f%s`6's Stats...`n",Player[ActivePlayer].name);
	od_set_attrib(0x0e);
	j_printf("`e-==-==-==-==-==-==-==-==-==-==-==-==-==-==-==-==-==-==-`n");
	od_set_cursor(3,5);
	j_printf("`6Experience`8......`e%ld",Player[ActivePlayer].experience);
	od_set_cursor(3,45);
	j_printf("`6Level`8...........`e%d",Player[ActivePlayer].level);
	od_set_cursor(4,5);
	j_printf("`6Health Points`8...`6(`e%d`6 of `e%d`6)",Player[ActivePlayer].hp_cur,Player[ActivePlayer].hp_max);
	od_set_cursor(4,45);
	j_printf("`6Magic Points`8....`6(`e%d`6 of `e%d`6)",Player[ActivePlayer].magic_cur,Player[ActivePlayer].magic_max);
	od_set_cursor(5,5);
	j_printf("`6Gold In Hand`8....`e%ld",Player[ActivePlayer].gold_hand);
	od_set_cursor(5,45);
	j_printf("`6Gold In Bank`8....`e%ld",Player[ActivePlayer].gold_bank);
	od_set_cursor(6,5);
	j_printf("`6Weapon Used`8.....`e%s",weapon_name[Player[ActivePlayer].weapon]);

	if (Player[ActivePlayer].condition==0)
	{
		od_set_cursor(6,45);
		j_printf("`6Weapon Strength`8.`e%d",Player[ActivePlayer].attack_str+weapon_strength[Player[ActivePlayer].weapon]);
	}
	else
	{
		od_set_cursor(6,45);
		temp=Player[ActivePlayer].attack_str+weapon_strength[Player[ActivePlayer].weapon];
		temp+=(condition_adjust_strength[Player[ActivePlayer].condition])*Player[ActivePlayer].level;
		j_printf("`6Weapon Strength`8.`a%d",temp);
	}
	od_set_cursor(7,5);
	j_printf("`6Armor Used`8......`e%s",armor_name[Player[ActivePlayer].armor]);

	if (Player[ActivePlayer].condition==0)
	{
		od_set_cursor(7,45);
		j_printf("`6Armor Strength`8..`e%d",Player[ActivePlayer].defense_str+armor_strength[Player[ActivePlayer].armor]);
	}
	else
	{
		od_set_cursor(7,45);
		temp=Player[ActivePlayer].defense_str+armor_strength[Player[ActivePlayer].armor];
		temp+=(condition_adjust_defense[Player[ActivePlayer].condition])*Player[ActivePlayer].level;
		j_printf("`6Armor Strength`8..`a%d",temp);
	}
	od_set_cursor(8,5);
	j_printf("`6Dungeon Fights`8..`e%d",Player[ActivePlayer].dungeon_fights);
	od_set_cursor(8,45);
	j_printf("`6Player Fights`8...`e%d",Player[ActivePlayer].player_fights);
	if (!PG)
	{
		if (Player[ActivePlayer].lays>0)
		{
			od_set_cursor(9,5);
			j_printf("`6Lays`8............`e%d",Player[ActivePlayer].lays);
		}
		else
		{
			od_set_cursor(9,5);
			j_printf("`6Lays`8............`dVIRGIN");
		}
	}
	od_set_cursor(9,45);
	if (Player[ActivePlayer].sex==0)
		j_printf("`6Sex`8.............`eMALE");
	else
		j_printf("`6Sex`8.............`bFEMALE");
	od_set_cursor(10,5);
	j_printf("`6Times Knighted`8..`e%d",Player[ActivePlayer].knighted);
	od_set_cursor(10,45);
	if (Player[ActivePlayer].alive==TRUE)
		j_printf("`6Health Status`8...`cALIVE");
	else
		j_printf("`6Health Status`8...`cDEAD");

	if (Player[ActivePlayer].condition==0)
	{
		od_set_cursor(11,5);
		j_printf("`6Charm`8...........`e%d",Player[ActivePlayer].charm);
	}
	else
	{
		od_set_cursor(11,5);
		temp=Player[ActivePlayer].charm;
		temp+=(condition_adjust_charm[Player[ActivePlayer].condition])*Player[ActivePlayer].level;
		j_printf("`6Charm`8...........`a%d",temp);
	}
	od_set_cursor(11,45);
	j_printf("`6Condition`8.......`a%s",condition_string[Player[ActivePlayer].condition]);

	od_set_cursor(12,5);
	j_printf("`6Sons`8............`e%d",Player[ActivePlayer].sons);
	od_set_cursor(12,45);
	j_printf("`6Daughters`8.......`e%d",Player[ActivePlayer].daughters);
	od_set_cursor(13,5);
	if (Player[ActivePlayer].married_to==-1)
		j_printf("`6Marital Status`8..`4SINGLE");
	if (Player[ActivePlayer].married_to>=0)
		j_printf("`6Marital Status`8..`4married to `c%s",Player[Player[ActivePlayer].married_to].name);
	if (Player[ActivePlayer].married_to==-2)
	{
		if (Player[ActivePlayer].sex==0)
			j_printf("`6Marital Status`8..`4married to `cBETTY");
		if (Player[ActivePlayer].sex==1)
			j_printf("`6Marital Status`8..`4married to `cBILLY");
	}

	j_printf("`n");
}





/***********************************
 *    KING'S CROWN ADDON SOURCE    *
 ***********************************/
//    FUNCTION:
//        void j_printf(char *fmt, ...);
//    DESCRIPTION:
//        this function works pretty much like the standard od_printf with
//        the following exceptions:
//          `1 = change to dk. blue
//          `2 = change to dk. green
//          `3 = change to dk. cyan
//          `4 = change to dk. red
//          `5 = change to dk. magenta
//          `6 = change to brown (dk. yellow)
//          `7 = change to lt. grey (dk. white)
//          `8 = change to dk. grey (lt. black)
//          `9 = change to lt. blue
//          `a = change to lt. green
//          `b = change to lt. cyan
//          `c = change to lt. red
//          `d = change to lt. magenta
//          `e = change to yellow
//          `f = change to white
//          `t = four spaces
//          `n = \n\r combination
//          %s = display string (uses color codes)
//          %l = displays unsigned long integer (with commas)
//    RETURNS:
//        nothing
void j_printf(char *fmt, ...)
{
	va_list ap;         /* Points to each unnamed arg in turn */
	char *p, *sval,cval;
	int ival;
	Ulong uval;
	double dval;

	va_start(ap, fmt);      /* make ap point to 1st unnamed arg */
	for (p =  fmt; *p; p++)
	{
		if (*p == '`')
		{

			switch(*++p)
			{
				case 't':
					od_printf("    ");
					break;

				case 'n':
					od_printf("\n\r");
					break;

				//case '0':
				//    od_printf("`black`");
				//    break;

				case '1':
					od_printf("`blue`");
					break;

				case '2':
					od_printf("`green`");
					break;

				case '3':
					od_printf("`cyan`");
					break;

				case '4':
					od_printf("`red`");
					break;

				case '5':
					od_printf("`magenta`");
					break;

				case '6':
					od_printf("`brown`");
					break;

				case '7':
					od_printf("`white`");
					break;

				case '8':
					od_printf("`bright black`");
					break;

				case '9':
					od_printf("`bright blue`");
					break;

				case 'a':
					od_printf("`bright green`");
					break;

				case 'b':
					od_printf("`bright cyan`");
					break;

				case 'c':
					od_printf("`bright red`");
					break;

				case 'd':
					od_printf("`bright magenta`");
					break;

				case 'e':
					od_printf("`bright brown`");
					break;

				case 'f':
					od_printf("`bright white`");
					break;

				default:
					od_putch(*p);
					break;

			}
			continue;
		}
		if (*p != '%')
		{
			od_putch(*p);
			continue;
		}
		switch (*++p)
		{
			case 'c':
				cval=va_arg(ap, char);
				od_putch(cval);
				break;
			case 'd':
				ival=va_arg(ap, int);
				od_printf("%d",ival);
				break;

			case 'f':
				dval=va_arg(ap, int);
				od_printf("%f",dval);
				break;

			case 's':
				for (sval = va_arg(ap, char *); *sval; sval++)
				{
					if (*sval=='`')
					{
						sval++;
						if (*sval>='1' && *sval<='9')
						{
							od_set_attrib(*sval-'0');
						}
						if (*sval>='a' && *sval<='f')
						{
							od_set_attrib((*sval-'a')+10);
						}
						if (*sval=='t')
						{
							od_printf("    ");
						}
						if (*sval=='n')
						{
							od_printf("\n\r");
						}
					}
					else
					{
						od_putch(*sval);
					}
				}
				break;

			case 'l':
				++p;
				j_putUlong(va_arg(ap, unsigned long));
				break;

			default:
				od_putch(*p);
				break;
		}
	}
	va_end(ap);
}


//    FUNCTION:
//        void j_putUlong(Ulong Number);
//    DESCRIPTION:
//        This function displays the contents of an unsigned long integer.
//        Commas are also displayed in correct position (ie, every 3rd #).
//    RETURNS:
//        nothing
void j_putUlong(Ulong Number)
{
	int t,f;

	f=0;
	for (t=0;t<10;t++)
	{
		if (Number/NumBase[t] > 0 || t==9)
			f=1;

		if (f==1)
		{
			j_printf("%c",Hex[Number/NumBase[t]]);
			if (t==0 || t==3 || t==6)
				j_printf(",");

			Number%=NumBase[t];
		}
	}
}
//    FUNCTION:
//        void a_putUlong(Ulong Number);
//    DESCRIPTION:
//        This function displays the contents of an unsigned long integer.
//        Commas are also displayed in correct position (ie, every 3rd #).
//        output goes only to the local display.
//    RETURNS:
//        nothing
void a_putUlong(Ulong Number)
{
	int t,f;
	char num[2];

	f=0;
	for (t=0;t<10;t++)
	{
		if (Number/NumBase[t] > 0 || t==9)
			f=1;

		if (f==1)
		{
			sprintf(num,"%c",Hex[Number/NumBase[t]]);
			_outtext(num);
			if (t==0 || t==3 || t==6)
			{
				_outtext(",");
			}

			Number%=NumBase[t];
		}
	}
}

//    FUNCTION:
//        Ulong FInputString(char *string,int maxlen);
//    DESCRIPTION:
//        retreives an unsigned long from the user as input.
//        the first array must be defined externally.  like a standard
//        input buffer. Commas are stripped from input here.
//    RETURNS:
//        the Unsigned Long value entered by the user.
Ulong FInputString(char *string,int maxlen)
{
	char U,t;

	t=0;

	do
	{
		U=od_get_key(TRUE);
		if (t<maxlen && U!=8)
			j_printf("%c",U);
		if (U==8)           //backspace
		{
			if (t>0)
			{
				t--;
				j_printf("\b \b");
			}
		}
		else if (t<maxlen && U!=',')
		{
			string[t++]=U;
		}
	} while (U!=13);
	string[t--]=0;
	return((Ulong)atol(string));
}

//    FUNCTION:
//        Uint GetUint(char Attrib, int length);
//    DESCRIPTION:
//        user enters an unsigned integer of no more than length characters.
//        entered characters will be of Attrib color.
//    RETURNS:
//        the Unsigned integral value entered by the user.
Uint GetUint(char Attrib, int length)
{
	char temp[20];

	od_set_attrib(Attrib);
	return((Uint)FInputString(temp,length));
}

//    FUNCTION:
//        Uint GetUint(char Attrib, int length);
//    DESCRIPTION:
//        user enters an unsigned long of no more than length characters.
//        entered characters will be of Attrib color.
//    RETURNS:
//        the Unsigned long value entered by the user.
Ulong GetUlong(char Attrib, int length)
{
	char temp[20];

	od_set_attrib(Attrib);
	return((Ulong)FInputString(temp,length));
}

//    FUNCTION:
//        Uint ReadKCtime(void);
//    DESCRIPTION:
//        reads KCDROP.DAT and updates the users time limit.
//    RETURNS:
//        the Unsigned int value representing the users time left.
Uint ReadKCtime(void)
{
	FILE *fp;
	char S[81],F[81];

	sprintf(F,"%sKCDROP.DAT",GameDir);

	fp=fopen(F,"r");
	if (fp)
	{
		do
		{
			if (!feof(fp))
				fgets(S,80,fp);
		} while (strcmp(S,"time left:\n")!=0 && !feof(fp));
		fscanf(fp,"%ld",&od_control.user_timelimit);
		fclose(fp);
	}
	return(od_control.user_timelimit);
}

//    FUNCTION:
//        Uint ReadKCplayer(void);
//    DESCRIPTION:
//        reads KCDROP.DAT and records the current
//        users index value (ActivePlayer).
//    RETURNS:
//        the Unsigned int value representing the users index.
Uint ReadKCplayer(void)
{
	FILE *fp;
	char S[81],F[81];

	LoadPlayers();

	sprintf(F,"%sKCDROP.DAT",GameDir);

	fp=fopen(F,"r");
	if (fp)
	{
		do
		{
			if (!feof(fp))
				fgets(S,80,fp);
		} while (strcmp(S,"active player:\n")!=0 && !feof(fp));
		fscanf(fp,"%ld",&ActivePlayer);
		fclose(fp);
	}
	strcpy(od_control.user_name,Player[ActivePlayer].user_name);
	od_set_statusline(STATUS_NONE);
	od_set_statusline(STATUS_NORMAL);
	return(ActivePlayer);
}

//    FUNCTION:
//        void LoadPlayer(int Index);
//    DESCRIPTION:
//        This function will load a given player record, referenced by
//        the Index parameter.  Use this function to load only
//        one characters information.
//    RETURNS:
//        nothing
void LoadPlayer(int Index)
{
	int fp;
	int t,a1,d1;
	char fn[81];


	sprintf(fn,"%sPLAYER.DAT",GameDir);
	fp=open(fn,O_RDONLY|O_BINARY);

	lseek (fp,(long int)(sizeof(USER_CHARACTER) * Index),SEEK_SET);
	read (fp,&Player[Index].name[0],sizeof(USER_CHARACTER));
	close(fp);
}

//    FUNCTION:
//        void SavePlayer(int Index);
//    DESCRIPTION:
//        This function will save a given player record, referenced by
//        the Index parameter. Use this function to save only
//        one characters infomation.
//    RETURNS:
//        nothing
void SavePlayer(int Index)
{
	int fp;
	char fn[81];
	int t,a1,d1;

	sprintf(fn,"%sPLAYER.DAT",GameDir);
	do
	{
		fp=open (fn,O_WRONLY|O_BINARY);
	} while (fp==-1);

	lseek (fp,(long int)(sizeof(USER_CHARACTER) * Index),SEEK_SET);

	write (fp,&Player[Index].name[0],sizeof(USER_CHARACTER));
	close(fp);
}

//    FUNCTION:
//        void LoadPlayers(void);
//    DESCRIPTION:
//        This function will load all player records.
//        Use this function to load all of the characters information.
//        NOTE: this function will create a new PLAYER.DAT if none exist.
//    RETURNS:
//        nothing
void LoadPlayers(void)
{
	FILE *fp;
	int t;
	char fn[81];

	sprintf(fn,"%sPLAYER.DAT",GameDir);

	fp=fopen (fn,"rb");
	if (fp==NULL)       //if no PLAYER.DAT file exists, make one
	{
		int t;

		for (t=0;t<MAXPL;t++)
		{
			Player[t].name[0]=0;
			Player[t].attack_str=0;
			Player[t].defense_str=0;
			Player[t].hp_max=0;
			Player[t].hp_cur=0;
			Player[t].magic_max=0;
			Player[t].magic_cur=0;
			Player[t].gold_hand=0;
			Player[t].gold_bank=0;
			Player[t].experience=0;
			Player[t].level=0;
			Player[t].lays=0;
			Player[t].weapon=0;
			Player[t].armor=0;
			Player[t].user_name[0]=0;
			Player[t].sc_light=0;
			Player[t].sc_healing=0;
			Player[t].sc_map=0;
			Player[t].sc_teleport=0;
			Player[t].sc_disarm=0;
			Player[t].sc_speed=0;
			Player[t].sc_invisibility=0;
			Player[t].sc_enchant=0;
			Player[t].sc_levitate=0;            //number of levitate Scrolls
			Player[t].sc_return=0;          //number of Rune of Return scrolls
			Player[t].dungeon_fights=0;
			Player[t].player_fights=0;
			Player[t].sex=0;                //0=male   1=female
			Player[t].knighted=0;
			Player[t].alive=0;
			Player[t].charm=0;              //characters charm
			Player[t].dungeon_level=0;      //current dungeon level (0=none)
			Player[t].dungeon_seed=0;       //seed of current dungeon
			Player[t].dungeon_xpos=0;       //horizontal position in dungeon
			Player[t].dungeon_ypos=0;       //vertical position in dungeon
			Player[t].dungeon_max=0;        //maximum dungeon level attained so far
			Player[t].last_on=1;
			Player[t].condition=0;          //drunkeness level
			Player[t].location=3;           //0=camped, 1=inn, 2=dungeon, 3=online
			Player[t].flirt_inn=0;          //0=has not flirted at inn
			Player[t].flirt_player=0;       //0=has not flirted with other player
			Player[t].heard_bards=0;        //0=has not heard the bards jam yet
			Player[t].married_to=-1;        //-1= not married, -2 = married to Billy/Betty
			Player[t].sons=0;               //number of sons
			Player[t].daughters=0;          //number of daughters
			Player[t].kings_crown=0;        //player has the crown?
		}
		do
		{
			fp=fopen(fn,"wb");
		} while (!fp);

		fwrite(Player, sizeof(USER_CHARACTER), MAXPL,fp);
		fclose(fp);
	}
	else
	{
		fread (Player,sizeof(USER_CHARACTER), MAXPL,fp);
		fclose(fp);

	}
}

//    FUNCTION:
//        void SavePlayers(void);
//    DESCRIPTION:
//        This function will save all player records.
//        Use this function to save all of the characters information.
//    RETURNS:
//        nothing
void SavePlayers(void)
{
	FILE *fp;
	char fn[81];
	int t;

	sprintf(fn,"%sPLAYER.DAT",GameDir);

	do
	{
		fp=fopen (fn,"wb");
	} while (!fp);

	fwrite(Player, sizeof(USER_CHARACTER), MAXPL,fp);
	fclose(fp);
}



//    FUNCTION:
//        void LoadGameDir(int Node);
//    DESCRIPTION:
//        This function loads the GameDir string.
//        It reads the current directory for the KING#.CFG files where #
//        is the current Node.  This function should be called before
//        any other in this library as some of the other functions require
//        the use of the GameDir string.
//    RETURNS:
//        nothing
void LoadGameDir(int Node)
{
	FILE *fp;
	char fn[161];
	int t;
	int DF;
	int PF;
	int ML;
	Uint RNUM;
	Uint GNUM;
	long ID;                   //inactivity delete # of days

	if (Node>-1)
	{
		sprintf(fn,"KING%d.CFG",Node);

		fp=fopen(fn,"r");
		if (fp)
		{
			fscanf(fp,"%u",&GNUM);
			fscanf(fp,"%u",&RNUM);
			fscanf(fp,"%d",&DF);
			fscanf(fp,"%d",&PF);
			fscanf(fp,"%d",&ML);
			fscanf(fp,"%d",&ID);
			fgets(GameDir,80,fp);
			fclose(fp);
		}
	}
	if (GameDir[0]=='\n')
		sprintf(GameDir,"");
	for (t=0;GameDir[t];t++)
	{
		if (isspace(GameDir[t]))
		{
			GameDir[t]=0;
			break;
		}
	}
}

//    FUNCTION:
//        void AddNews(char * string);
//    DESCRIPTION:
//        This function will allow you to add news to King's Crown.  It will
//        Append the NULL-TERMINATED string that is pointed to by the
//        parameter to the end of TNEWS.DAT
//    RETURNS:
//        nothing
void AddNews(char *string)
{
	FILE *fp;
	char fn[81];

	sprintf(fn,"%stnews.dat",GameDir);
	fp=fopen(fn,"a");
	if (fp)
	{
		fputs(string,fp);
		fputc(0, fp);
		fclose(fp);
	}
}

//    FUNCTION:
//        void SendMail(int FIndex, int TIndex, char *Mail, int flirt);
//    DESCRIPTION:
//        This function sends mail to a player.
//        FIndex is the index of the player sending the mail.
//            (-1 means "sent from nobody")
//        TIndex is the index of the player that should receive the mail.
//        *Mail is a pointer to a NULL-TERMINATED string containing the
//            message to be sent.
//        flirt should be set to 0.
//    RETURNS:
//        nothing
void SendMail(int FIndex, int TIndex, char *Mail, int flirt)
{
	FILE *fp;
	char FN[81];

	sprintf(FN,"%sMAIL%d.DAT",GameDir,TIndex);

	fp=fopen(FN,"a");
	fprintf(fp,"%d\n",FIndex);
	fprintf(fp,"%d",flirt);
	fputs(Mail,fp);
	fputc(0,fp);
	fclose(fp);
}

//    FUNCTION:
//        void LoadDropPath(void);
//    DESCRIPTION:
//        This function reads the file called "DROPLIST.CFG" and places the
//        drop file infomation into the DropPath arrays.
//        This function also reads the system info
//        and stores those values for each node into the SYSINFO array.
//        0=runs on this system
//        1=runs on a lan station
//        the DropPath and SYSINFO arrays are to be indexed by the current
//        node.
//    RETURNS:
//        nothing
void LoadDropPath(void)
{
	int t;
	FILE *fp;
	char fn[81];

	sprintf(fn,"%sDROPLIST.CFG",GameDir);
	fp=fopen(fn,"r");
	if (fp)
	{
		for (t=0;t<MAXNODES;t++)
		{
			fscanf(fp,"%d",&SYSTYPE[t]);
			fscanf(fp,"%s",DropPath[t]);
		}
		fclose(fp);
	}
}

//    FUNCTION:
//        Uint FindPlayer(char *UserName);
//    DESCRIPTION:
//        This function will look through the Player.Dat for that users player
//        record.
//    RETURNS:
//        MAXPL  if all entries are used and the users character was not found
//        Index of the users character
//        Index of the first empty slot if users character was not found
Uint FindPlayer(char *UserName)
{
	int t,f;
	char *S;

	LoadPlayers();
	f=strlen(UserName)-1;

	for (t=f;UserName[t]==' ';)
		UserName[t--]=0;

	for (t=0;t<MAXPL;t++)
	{
		if (strcmp(UserName,Player[t].user_name) == 0)
		{
			return(t);
		}
	}

	f=0;
	for (t=0;t<MAXPL;t++)
		if (Player[t].name[0]==0)
			return(t);

	return(MAXPL);            //return error, too many players!!!
}











/*********************************************************
 *                                                       *
 *  some notes about making King's Crown Addon Programs  *
 *                                                       *
 *********************************************************

	This source, was generated for use with Microsoft Quick C 2.5.
	Microsoft C/C++ should work with no changes.  Use with other compilers
	may need changes to this code.  Brian Pirie's OPENDOORS is pretty much
	required for its use.

	I offer no guarentees that any of this code will actually compile and
	work with your addons.  It does, however, compile on my system, and
	was used in creating "CURLYS CASINO". Also there are no guarentees
	that this code is BUGFREE.  I will not be held resposible for any damage
	done while using anything contained in this listing.

	KING.EXE will generate KCDROP.DAT (drop file) prior to calling any addon.
	please examine KCDROP.DAT for the syntax of that file.  it is pretty much
	self explanitory.

	When KING.EXE calls the Addon Program(IGM), it will pass the following
	information to the IGM:

		PROGNAME.EXE # /RIP /LOCALRIP /PG /MAINT

			#         is the Current Node.
			/RIP      if RIPterm is being used.
			/LOCALRIP if the sysop has that enabled.
			/PG       if PG mode is enabled (disabling getting laid)
			/MAINT    when KING.EXE is running maintenence.

	Note:  The Node # will ALWAYS be the first argument.
		   /RIP,/LOCALRIP,/PG, and /MAINT may be passed in any order.


	/MAINT is required and should be accounted for even if your IGM
	doesn't need to run maintenence.  In this case it should do nothing
	and just exit (do nothing) if the /MAINT parameter is detected.
	If your IGM needs maintenence the it should only run maintenence when
	/MAINT is detected.


	/RIP,/LOCALRIP, and /PG  are all optional.

	before calling the IGM, King's Crown writes a file called "KCDROP.DAT"
	that contains some com port information.

	this source is released as FreeWare, with the following conditions:
		1) any King's Crown Addon should be uploaded to my BBS for reviewing.
		2) any registration fees should be free for the authors BBS use,
		   again for reviewing purposes only.
		3) I (the author) should be notified of any and all releases of
		   addons made for use with King's Crown.
		4) authors writing KC addons using OPENDOORS should register OPENDOORS
		   prior to releasing any addons.  It is after all a shareware product
		   and Brian Pirie deserves to get paid for his excellent work.
		5) King's Crown should be registered prior to release of any addon
		   created for it.  You will need to have a registered copy of King's
		   Crown to test the addon remotely anyways.

	Additional help MAY be aquired from my BBS:

		Hippie's House BBS
		(206)-639-8578 node 1
		(206)-639-2227 node 2
		(206)-639-2228 node 3

				or

		Longhair@concentric.net




	ONE FINAL NOTE:
		good luck...

******************************************************************/
